/**
 * @file simple.c
 * Simple test for SE with 1/256.. 1/255.. 1/254.. probability of success
 *
 * Copyright (c) 2014 UC Berkeley
 * @author Mathias Payer <mathias.payer@nebelwelt.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 */


#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/mman.h>

#if defined(KLEE)
#include <klee/klee.h>
#endif /* KLEE */

__attribute__ ((noinline))
void foo() {
    printf("Starting\n");
}

__attribute__ ((noinline))
void bar() {
    printf("Done\n");
}

#define MAGIC 42
int main(int argc, char *argv[]) {
    unsigned char *buf0;
    unsigned int i;

    if (argc != 1) {
        printf("Usage: %s \n", argv[0]);
        return -1;
    }

    // initialize symbolic memory for KLEE / FuzzBALL
#if defined(KLEE)
    buf0 = (unsigned char*)malloc(4096);
#else
    buf0 = (unsigned char*)mmap((void *)0xc0fe0000, 4096,
                                PROT_READ|PROT_WRITE,
                                MAP_PRIVATE|MAP_ANONYMOUS|MAP_FIXED, -1, 0);
    assert(((long)buf0) != -1);
#endif
    
#if defined(KLEE)
    // KLEE needs explicit calls to make memory symbolic
    unsigned char *symb_buf = (unsigned char*)malloc(1);
    klee_make_symbolic(symb_buf, 1, "input buffer");
    for (i = 0; i < 1; ++i)
        buf0[i] = symb_buf[i];
#endif  /* KLEE */
    // add some concrete values into the process memory
    char arr[256];
    for (i=0; i<256; i++)
        arr[i] = i;

    // convenience function to grep for where SE starts
    foo();

    // buf0[0] is symbolic
    // copy the concrete value from arr[symb] into buf0[42]
    buf0[42] = arr[buf0[0]];

    // check if the copied value was the magic value defined above
    if (buf0[42] == MAGIC) {
        printf("Correctly recovered value 42\n");
    }

    // convenience function to grep for where SE stops
    bar();

    // tear down for KLEE / FuzzBALL
#if defined(KLEE)
    klee_quit();
    printf("Opps, did not work\n");
#endif  /* KLEE */
#if defined(KLEE)
    free(buf0);
#else  /* KLEE/not KLEE */
    munmap(buf0, 4096);
#endif  /* not KLEE */
    return 0;
}
